# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 Domsense s.r.l. (<http://www.domsense.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
import math
import decimal_precision as dp

class account_vat_period_end_statement(osv.osv):

    def _compute_authority_vat_amount(self, cr, uid, ids, field_name, arg, context):
        res={}
        for i in ids:
            statement = self.browse(cr, uid, i)
            debit_vat_amount = 0.0
            credit_vat_amount = 0.0
            generic_vat_amount = 0.0
            for debit_line in statement.debit_vat_account_line_ids:
                debit_vat_amount += debit_line.amount
            for credit_line in statement.credit_vat_account_line_ids:
                credit_vat_amount += credit_line.amount
            for generic_line in statement.generic_vat_account_line_ids:
                generic_vat_amount += generic_line.amount
            authority_amount = (- debit_vat_amount - credit_vat_amount - generic_vat_amount
                - statement.previous_credit_vat_amount - statement.previous_debit_vat_amount)
            res[i] = authority_amount
        return res

    def _compute_payable_vat_amount(self, cr, uid, ids, field_name, arg, context):
        res={}
        for i in ids:
            statement = self.browse(cr, uid, i)
            debit_vat_amount = 0.0
            for debit_line in statement.debit_vat_account_line_ids:
                debit_vat_amount += debit_line.amount
            res[i] = debit_vat_amount
        return res

    def _compute_deductible_vat_amount(self, cr, uid, ids, field_name, arg, context):
        res={}
        for i in ids:
            statement = self.browse(cr, uid, i)
            credit_vat_amount = 0.0
            for credit_line in statement.credit_vat_account_line_ids:
                credit_vat_amount += credit_line.amount
            res[i] = credit_vat_amount
        return res

    _name = "account.vat.period.end.statement"
    _rec_name = 'period_id'
    _columns = {
        'debit_vat_account_line_ids': fields.one2many('statement.debit.account.line', 'statement_id', 'Debit VAT', help='The accounts containing the debit VAT amount to write-off', required=True, states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),

        'credit_vat_account_line_ids': fields.one2many('statement.credit.account.line', 'statement_id', 'Credit VAT', help='The accounts containing the credit VAT amount to write-off', required=True, states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),

        'previous_credit_vat_account_id': fields.many2one('account.account', 'Previous Credits VAT', help='Credit VAT from previous periods', required=True, states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),
        'previous_credit_vat_amount': fields.float('Previous Credits VAT Amount', states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),

        'previous_debit_vat_account_id': fields.many2one('account.account', 'Previous Debits VAT', help='Debit VAT from previous periods', states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),
        'previous_debit_vat_amount': fields.float('Previous Debits VAT Amount', states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),
        'previous_debit_vat_threshold': fields.float('Previous Debits VAT Threshold', states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}, help='Threshold used to establish whether debit has to be registered on the authority VAT account or the previous debits VAT account'),

        'generic_vat_account_line_ids': fields.one2many('statement.generic.account.line', 'statement_id', 'Other VAT Credits / Debits or Tax Compensations', states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),

        'authority_vat_account_id': fields.many2one('account.account', 'Tax Authority VAT Account', required=True, states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),
        'authority_vat_amount': fields.function(_compute_authority_vat_amount, method=True, string='Amount to pay'),
        'payable_vat_amount': fields.function(_compute_payable_vat_amount, method=True, string='Payable VAT Amount'),
        'deductible_vat_amount': fields.function(_compute_deductible_vat_amount, method=True, string='Deductible VAT Amount'),

        'journal_id': fields.many2one('account.journal', 'Journal', required=True, states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),
        'period_id': fields.many2one('account.period', 'Period', required=True, states={'confirmed': [('readonly', True)], 'paid': [('readonly', True)]}),
        'move_id': fields.many2one('account.move', 'VAT statement move', readonly=True),

        'state': fields.selection([
            ('draft', 'Draft'),
            ('confirmed', 'Confirmed'),
            ('paid', 'Paid'),
            ], 'State', readonly=True),
    }

    _defaults = {
        'period_id': lambda self, cr, uid, c: self.pool.get('account.period').find(cr, uid, context=c)[0],
    }

    def _get_tax_code_amount(self, cr, uid, tax_code_id, period_id, context):
        if not context:
            context={}
        context['period_id'] = period_id
        return self.pool.get('account.tax.code').browse(cr, uid, tax_code_id, context)._sum_period(
            None, None, context)[tax_code_id]

    def unlink(self, cr, uid, ids, context=None):
        if isinstance(ids, (long,int)):
            ids = [ids]
        for statement in self.browse(cr, uid, ids, context):
            if statement.state == 'confirmed':
                raise osv.except_osv(_('Error!'), _('You cannot delete a confirmed statement'))
        res = super(account_vat_period_end_statement, self).unlink(cr, uid, ids, context)
        return res

    def on_change_previous_credit_vat_account_id(self, cr, uid, ids, previous_credit_vat_account_id=False, context=None):
        res = {}
        res['value'] = {}
        if not previous_credit_vat_account_id:
            return res
        res['value']['previous_credit_vat_amount'] = self.pool.get('account.account').browse(cr, uid,
            previous_credit_vat_account_id, context).balance
        return res

    def on_change_previous_debit_vat_account_id(self, cr, uid, ids, previous_debit_vat_account_id=False, context=None):
        res = {}
        res['value'] = {}
        if not previous_debit_vat_account_id:
            return res
        res['value']['previous_debit_vat_amount'] = self.pool.get('account.account').browse(cr, uid,
            previous_debit_vat_account_id, context).balance
        return res

    def statement_draft(self, cr, uid, ids, context=None):
        for statement in self.browse(cr, uid, ids, context):
            if statement.move_id:
                statement.move_id.unlink()
        self.write(cr, uid, ids , {'state': 'draft'})

    def create_move(self, cr, uid, ids, context=None):
        move_obj = self.pool.get('account.move')
        line_obj = self.pool.get('account.move.line')
        for statement in self.browse(cr, uid, ids, context):

            statement_ids = self.search(cr, uid, [('period_id', '=', statement.period_id.id)])
            if len(statement_ids) > 1:
                raise osv.except_osv(_('Error!'), _('The current period has VAT statement yet'))

            move_data = {
                'name': _('VAT statement') + ' - ' + statement.period_id.name,
                'period_id': statement.period_id.id,
                'date': statement.period_id.date_stop,
                'journal_id': statement.journal_id.id,
                }
            move_id = move_obj.create(cr, uid, move_data)
            statement.write({'move_id': move_id})

            for debit_line in statement.debit_vat_account_line_ids:
                debit_vat_data = {
                    'name': _('Debit VAT'),
                    'account_id': debit_line.account_id.id,
                    'move_id': move_id,
                    'period_id': statement.period_id.id,
                    'journal_id': statement.journal_id.id,
                    'debit': 0.0,
                    'credit': 0.0,
                    'date': statement.period_id.date_stop,
                    }
                if debit_line.amount < 0:
                    debit_vat_data['debit'] = math.fabs(debit_line.amount)
                else:
                    debit_vat_data['credit'] = math.fabs(debit_line.amount)
                line_obj.create(cr, uid, debit_vat_data)

            for credit_line in statement.credit_vat_account_line_ids:
                credit_vat_data = {
                    'name': _('Credit VAT'),
                    'account_id': credit_line.account_id.id,
                    'move_id': move_id,
                    'period_id': statement.period_id.id,
                    'journal_id': statement.journal_id.id,
                    'debit': 0.0,
                    'credit': 0.0,
                    'date': statement.period_id.date_stop,
                    }
                if credit_line.amount < 0:
                    credit_vat_data['debit'] = math.fabs(credit_line.amount)
                else:
                    credit_vat_data['credit'] = math.fabs(credit_line.amount)
                line_obj.create(cr, uid, credit_vat_data)

            if statement.previous_credit_vat_amount:
                previous_credit_vat_data = {
                    'name': _('Previous Credits VAT'),
                    'account_id': statement.previous_credit_vat_account_id.id,
                    'move_id': move_id,
                    'period_id': statement.period_id.id,
                    'journal_id': statement.journal_id.id,
                    'debit': 0.0,
                    'credit': 0.0,
                    'date': statement.period_id.date_stop,
                    }
                if statement.previous_credit_vat_amount < 0:
                    previous_credit_vat_data['debit'] = math.fabs(statement.previous_credit_vat_amount)
                else:
                    previous_credit_vat_data['credit'] = math.fabs(statement.previous_credit_vat_amount)
                line_obj.create(cr, uid, previous_credit_vat_data)

            if statement.previous_debit_vat_amount:
                previous_debit_vat_data = {
                    'name': _('Previous Debits VAT'),
                    'account_id': statement.previous_debit_vat_account_id.id,
                    'move_id': move_id,
                    'period_id': statement.period_id.id,
                    'journal_id': statement.journal_id.id,
                    'debit': 0.0,
                    'credit': 0.0,
                    'date': statement.period_id.date_stop,
                    }
                if statement.previous_debit_vat_amount < 0:
                    previous_debit_vat_data['debit'] = math.fabs(statement.previous_debit_vat_amount)
                else:
                    previous_debit_vat_data['credit'] = math.fabs(statement.previous_debit_vat_amount)
                line_obj.create(cr, uid, previous_debit_vat_data)

            for generic_line in statement.generic_vat_account_line_ids:
                generic_vat_data = {
                    'name': _('Other VAT Credits / Debits'),
                    'account_id': generic_line.account_id.id,
                    'move_id': move_id,
                    'period_id': statement.period_id.id,
                    'journal_id': statement.journal_id.id,
                    'debit': 0.0,
                    'credit': 0.0,
                    'date': statement.period_id.date_stop,
                    }
                if generic_line.amount < 0:
                    generic_vat_data['debit'] = math.fabs(generic_line.amount)
                else:
                    generic_vat_data['credit'] = math.fabs(generic_line.amount)
                line_obj.create(cr, uid, generic_vat_data)

            if statement.authority_vat_amount > 0:
                end_debit_vat_data = {
                    'move_id': move_id,
                    'period_id': statement.period_id.id,
                    'journal_id': statement.journal_id.id,
                    'debit': 0.0,
                    'credit': math.fabs(statement.authority_vat_amount),
                    'date': statement.period_id.date_stop,
                    }
                if statement.authority_vat_amount > statement.previous_debit_vat_threshold:
                    end_debit_vat_data['name'] = _('Tax Authority VAT')
                    end_debit_vat_data['account_id'] = statement.authority_vat_account_id.id
                else:
                    end_debit_vat_data['name'] = _('VAT Debits')
                    end_debit_vat_data['account_id'] = statement.previous_debit_vat_account_id.id
                line_obj.create(cr, uid, end_debit_vat_data)

            elif statement.authority_vat_amount < 0:
                previous_credit_vat_data = {
                    'name': _('VAT Credits'),
                    'account_id': statement.previous_credit_vat_account_id.id,
                    'move_id': move_id,
                    'period_id': statement.period_id.id,
                    'journal_id': statement.journal_id.id,
                    'debit': math.fabs(statement.authority_vat_amount),
                    'credit': 0.0,
                    'date': statement.period_id.date_stop,
                    }
                line_obj.create(cr, uid, previous_credit_vat_data)

        self.write(cr, uid, ids , {'state': 'confirmed'})

        return True

    def _create_statement_account_line(self, cr, uid, statement_ids, tax_code_type='debit', context=None):
        if not context:
            context={}
        tax_code_pool = self.pool.get('account.tax.code')
        line_obj = self.pool.get('statement.' + tax_code_type + '.account.line')
        for statement in self.browse(cr, uid, statement_ids):
            context['period_id'] = statement.period_id.id
            tax_code_ids = tax_code_pool.search(cr, uid, [
                ('vat_statement_account_id', '!=', False),
                ('vat_statement_type', '=', tax_code_type),
                ], context=context)
            for tax_code in tax_code_pool.browse(cr, uid, tax_code_ids, context=context):
                line_obj.create(cr, uid, {
                    'statement_id': statement.id,
                    'account_id': tax_code.vat_statement_account_id.id,
                    'amount': tax_code.sum_period,
                    })
        return True

    def load_debit_accounts(self, cr, uid, ids, context=None):
        if not context:
            context={}
        self._create_statement_account_line(cr, uid, ids, tax_code_type='debit', context=context)
        return True

    def load_credit_accounts(self, cr, uid, ids, context=None):
        if not context:
            context={}
        self._create_statement_account_line(cr, uid, ids, tax_code_type='credit', context=context)
        return True

    def open_chart_of_taxes(self, cr, uid, ids, context=None):
        result = {}
        if context is None:
            context = {}
        for statement in self.browse(cr, uid, ids, context):
            mod_obj = self.pool.get('ir.model.data')
            act_obj = self.pool.get('ir.actions.act_window')
            period_obj = self.pool.get('account.period')
            result = mod_obj.get_object_reference(cr, uid, 'account', 'action_tax_code_tree')
            id = result and result[1] or False
            result = act_obj.read(cr, uid, [id], context=context)[0]

            fiscalyear_id = statement.period_id.fiscalyear_id.id
            result['context'] = str({'period_id': statement.period_id.id, \
                                     'fiscalyear_id': fiscalyear_id, \
                                        'state': 'posted'})

            period_code = statement.period_id.code
            result['name'] += period_code and (':' + period_code) or ''
            result['nodestroy'] = True
        return result

account_vat_period_end_statement()

class statement_debit_account_line(osv.osv):
    _name='statement.debit.account.line'
    _columns = {
        'account_id': fields.many2one('account.account', 'Account', required=True),
        'statement_id': fields.many2one('account.vat.period.end.statement', 'VAT statement'),
        'amount': fields.float('Amount', digits_compute= dp.get_precision('Account'), required=True),
        }

    def on_change_vat_account_id(self, cr, uid, ids, vat_account_id=False, context=None):
        res = {}
        res['value'] = {}
        if not vat_account_id:
            return res
        res['value']['amount'] = self.pool.get('account.account').browse(cr, uid, vat_account_id,
            context).balance
        return res

statement_debit_account_line()

class statement_credit_account_line(osv.osv):
    _name='statement.credit.account.line'
    _columns = {
        'account_id': fields.many2one('account.account', 'Account', required=True),
        'statement_id': fields.many2one('account.vat.period.end.statement', 'VAT statement'),
        'amount': fields.float('Amount', digits_compute= dp.get_precision('Account'), required=True),
        }

    def on_change_vat_account_id(self, cr, uid, ids, vat_account_id=False, context=None):
        res = {}
        res['value'] = {}
        if not vat_account_id:
            return res
        res['value']['amount'] = self.pool.get('account.account').browse(cr, uid, vat_account_id,
            context).balance
        return res

statement_credit_account_line()

class statement_generic_account_line(osv.osv):
    _name='statement.generic.account.line'
    _columns = {
        'account_id': fields.many2one('account.account', 'Account', required=True),
        'statement_id': fields.many2one('account.vat.period.end.statement', 'VAT statement'),
        'amount': fields.float('Amount', digits_compute= dp.get_precision('Account'), required=True),
        }

    def on_change_vat_account_id(self, cr, uid, ids, vat_account_id=False, context=None):
        res = {}
        res['value'] = {}
        if not vat_account_id:
            return res
        res['value']['amount'] = self.pool.get('account.account').browse(cr, uid, vat_account_id,
            context).balance
        return res

statement_generic_account_line()

class account_tax_code(osv.osv):
    _inherit = "account.tax.code"
    _columns = {
        'vat_statement_account_id': fields.many2one('account.account', "Account used for VAT statement"),
        'vat_statement_type': fields.selection([('credit','Credit'),('debit','Debit')], 'Type', help="This establish whether amount will be loaded as debit or credit"),
        }
    _defaults = {
        'vat_statement_type': 'debit',
    }
account_tax_code()
